@@ -1,13 +1,13 @@
 """Base geometry class and utilities
 """
 
-from functools import wraps
 import sys
 import warnings
 
 from shapely.coords import CoordinateSequence
+from shapely.ftools import wraps
 from shapely.geos import lgeos
-from shapely.impl import DefaultImplementation
+from shapely.impl import DefaultImplementation, delegated
 from shapely import wkb, wkt
 
 GEOMETRY_TYPES = [
@@ -42,7 +42,10 @@ def geom_factory(g, parent=None):
     ob.__class__ = getattr(mod, geom_type)
     ob.__geom__ = g
     ob.__p__ = parent
-    ob._ndim = 2 # callers should be all from 2D worlds
+    if lgeos.methods['has_z'](g):
+        ob._ndim = 3
+    else:
+        ob._ndim = 2
     return ob
 
 def exceptNull(func):
@@ -94,10 +97,14 @@ class BaseGeometry(object):
     def _is_empty(self):
         return self.__geom__ in [EMPTY, None]
 
+    # a reference to the so/dll proxy to preserve access during clean up
+    _lgeos = lgeos
+
     def empty(self):
+        # TODO: defer cleanup to the implementation. We shouldn't be
+        # explicitly calling a lgeos method here.
         if not (self._owned or self._is_empty):
-            from shapely.geos import lgeos
-            lgeos.GEOSGeom_destroy(self.__geom__)
+            self._lgeos.GEOSGeom_destroy(self.__geom__)
         self.__geom__ = EMPTY
 
     def __del__(self):
@@ -155,9 +162,10 @@ class BaseGeometry(object):
     # Coordinate access
     # -----------------
 
-    @exceptNull
     def _get_coords(self):
         """Access to geometry's coordinates (CoordinateSequence)"""
+        if self.is_empty:
+            return []
         return CoordinateSequence(self)
 
     def _set_coords(self, ob):
@@ -181,7 +189,6 @@ class BaseGeometry(object):
     # Type of geometry and its representations
     # ----------------------------------------
 
-    @exceptNull
     def geometryType(self):
         return geometry_type_name(self._geom)
     
@@ -243,9 +250,14 @@ class BaseGeometry(object):
             
     @property
     def centroid(self):
-        """Returns the geometric center of the polygon"""
+        """Returns the geometric center of the object"""
         return geom_factory(self.impl['centroid'](self))
 
+    @delegated
+    def representative_point(self):
+        """Returns a point guaranteed to be within the object, cheaply."""
+        return geom_factory(self.impl['representative_point'](self))
+
     @property
     def convex_hull(self):
         """Imagine an elastic band stretched around the geometry: that's a 
@@ -291,6 +303,7 @@ class BaseGeometry(object):
             res = resolution
         return geom_factory(self.impl['buffer'](self, distance, res))
 
+    @delegated
     def simplify(self, tolerance, preserve_topology=True):
         """Returns a simplified geometry produced by the Douglas-Puecker 
         algorithm
@@ -411,6 +424,7 @@ class BaseGeometry(object):
     # Linear referencing
     # ------------------
 
+    @delegated
     def project(self, other, normalized=False):
         """Returns the distance along this geometry to a point nearest the 
         specified point
@@ -423,7 +437,8 @@ class BaseGeometry(object):
         else:
             op = self.impl['project']
         return op(self, other)
-
+           
+    @delegated
     def interpolate(self, distance, normalized=False):
         """Return a point at the specified distance along a linear geometry
         
@@ -468,8 +483,9 @@ class BaseMultipartGeometry(BaseGeometry):
         "Multi-part geometries do not provide a coordinate sequence")
 
     @property
-    @exceptNull
     def geoms(self):
+        if self.is_empty:
+            return []
         return GeometrySequence(self, self.shape_factory)
 
     def __iter__(self):
@@ -570,7 +586,6 @@ class HeterogeneousGeometrySequence(GeometrySequence):
         g._owned = True
         return g
 
-
 # Test runner
 def _test():
     import doctest
