@@ -3,18 +3,19 @@ Proxies for the libgeos_c shared lib, GEOS-specific exceptions, and utilities
 """
 
 import atexit
-import functools
 import logging
 import os
 import sys
-import time
 import threading
 import ctypes
-from ctypes import cdll, CDLL, PyDLL, CFUNCTYPE, c_char_p, c_void_p, string_at
+from ctypes import cdll, CDLL, CFUNCTYPE, c_char_p, c_void_p, string_at
 from ctypes.util import find_library
 
+import ftools
 from ctypes_declarations import prototype
 
+
+
 # Begin by creating a do-nothing handler and adding to this module's logger.
 class NullHandler(logging.Handler):
     def emit(self, record):
@@ -25,40 +26,50 @@ LOG.addHandler(NullHandler())
 # Find and load the GEOS and C libraries
 # If this ever gets any longer, we'll break it into separate modules
 
+def load_dll(libname, fallbacks=None):
+    lib = find_library(libname)
+    if lib is not None:
+        return CDLL(lib)
+    else:
+        if fallbacks is not None:
+            for name in fallbacks:
+                try:
+                    return CDLL(name)
+                except OSError:
+                    # move on to the next fallback
+                    pass
+        # the end
+        raise OSError(
+            "Could not find library %s or load any of its variants %s" % (
+                libname, fallbacks or []))
+       
 if sys.platform == 'linux2':
-    _lgeos = CDLL(find_library('geos_c'))
-    free = CDLL(find_library('c')).free
+    _lgeos = load_dll('geos_c', fallbacks=['libgeos_c.so.1', 'libgeos_c.so'])
+    free = load_dll('c').free
     free.argtypes = [c_void_p]
     free.restype = None
 
 elif sys.platform == 'darwin':
-    lib = find_library('geos_c')
-    if lib is None:
-        ## try a few more locations
-        lib_paths = [
+    alt_paths = [
             # The Framework build from Kyng Chaos:
             "/Library/Frameworks/GEOS.framework/Versions/Current/GEOS",
             # macports
             '/opt/local/lib/libgeos_c.dylib',
-        ]
-        for path in lib_paths:
-            if os.path.exists(path):
-                lib = path
-                break
-    if lib is None:
-        raise ImportError("Could not find geos_c library")
-    _lgeos = CDLL(lib)
-    free = CDLL(find_library('c')).free
+    ]
+    _lgeos = load_dll('geos_c', fallbacks=alt_paths)
+    free = load_dll('c').free
     free.argtypes = [c_void_p]
     free.restype = None
 
 elif sys.platform == 'win32':
     try:
-        local_dlls = os.path.abspath(os.__file__ + "../../../DLLs")
+        egg_dlls = os.path.abspath(os.path.join(os.path.dirname(__file__),
+                                     r"..\DLLs"))
+        wininst_dlls =  os.path.abspath(os.__file__ + "../../../DLLs")
         original_path = os.environ['PATH']
-        os.environ['PATH'] = "%s;%s" % (local_dlls, original_path)
+        os.environ['PATH'] = "%s;%s;%s" % (egg_dlls, wininst_dlls, original_path)
         _lgeos = CDLL("geos.dll")
-    except (ImportError, WindowsError):
+    except (ImportError, WindowsError, OSError):
         raise
     def free(m):
         try:
@@ -68,28 +79,14 @@ elif sys.platform == 'win32':
             pass
 
 elif sys.platform == 'sunos5':
-    # Try the major versioned name first, falling back on the unversioned
-    # name.
-    try:
-        _lgeos = CDLL('libgeos_c.so.1')
-    except (OSError, ImportError):
-        _lgeos = CDLL('libgeos_c.so')
-    except:
-        raise
+    _lgeos = load_dll('geos_c', fallbacks=['libgeos_c.so.1', 'libgeos_c.so'])
     free = CDLL('libc.so.1').free
     free.argtypes = [c_void_p]
     free.restype = None
 
 else: # other *nix systems
-    # Try the major versioned name first, falling back on the unversioned
-    # name.
-    try:
-        _lgeos = CDLL('libgeos_c.so.1')
-    except (OSError, ImportError):
-        _lgeos = CDLL('libgeos_c.so')
-    except:
-        raise
-    free = CDLL('libc.so.6').free
+    _lgeos = load_dll('geos_c', fallbacks=['libgeos_c.so.1', 'libgeos_c.so'])
+    free = load_dll('c', fallbacks=['libc.so.6']).free
     free.argtypes = [c_void_p]
     free.restype = None
 
@@ -231,6 +228,7 @@ class LGEOS14(LGEOSBase):
         self.methods['boundary'] = self.GEOSBoundary
         self.methods['buffer'] = self.GEOSBuffer
         self.methods['centroid'] = self.GEOSGetCentroid
+        self.methods['representative_point'] = self.GEOSPointOnSurface
         self.methods['convex_hull'] = self.GEOSConvexHull
         self.methods['distance'] = self.GEOSDistance
         self.methods['envelope'] = self.GEOSEnvelope
@@ -270,7 +268,7 @@ class LGEOS15(LGEOSBase):
         for key in filter(lambda x: not x.endswith('_r'), keys):
             if key + '_r' in keys:
                 reentr_func = getattr(self._lgeos, key + '_r')
-                attr = functools.partial(reentr_func, self.geos_handle)
+                attr = ftools.partial(reentr_func, self.geos_handle)
                 attr.__name__ = reentr_func.__name__
                 setattr(self, key, attr)
             else:
@@ -297,10 +295,13 @@ class LGEOS15(LGEOSBase):
               ):
             pred.func.errcheck = errcheck_predicate
 
+        self.GEOSisValidReason.func.errcheck = errcheck_just_free
+        
         self.methods['area'] = self.GEOSArea
         self.methods['boundary'] = self.GEOSBoundary
         self.methods['buffer'] = self.GEOSBuffer
         self.methods['centroid'] = self.GEOSGetCentroid
+        self.methods['representative_point'] = self.GEOSPointOnSurface
         self.methods['convex_hull'] = self.GEOSConvexHull
         self.methods['distance'] = self.GEOSDistance
         self.methods['envelope'] = self.GEOSEnvelope
@@ -349,8 +350,7 @@ class LGEOS16LR(LGEOS16):
     def __init__(self, dll):
         super(LGEOS16LR, self).__init__(dll)
 
-        self.GEOSisValidReason.func.errcheck = errcheck_just_free
-        
+        self.methods['parallel_offset'] = self.GEOSSingleSidedBuffer
         self.methods['project'] = self.GEOSProject
         self.methods['project_normalized'] = self.GEOSProjectNormalized
         self.methods['interpolate'] = self.GEOSInterpolate
